/*
 * LineBreak  ~  v.0.2  ~  NightOwl  ~  Ball class.
 */
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.util.Random;

public class Ball extends Primitive{
    private int radius;
    private int startx, starty;
    private double xcalc, ycalc, xdir, ydir, startxdir, startydir, speed;
    private boolean fixed;
    private Paddle paddle;
    private Random rand = new Random(System.currentTimeMillis());
    private int angle, startangle;

    public Ball(int x, int y, int angle, int radius, Color color){
        startx = x; starty = y; xcalc = x; ycalc = y;
        startangle = angle;
        setPosition(x, y);
        resetState();
        setRadius(radius);
        setColor(color);
        setSpeed(Const.BALL_SPEED);
        setAngle(angle);
    }
    // get
    public int getRadius(){ return radius; }
    public boolean isFixed(){ return fixed; }
    public double getSpeed(){ return speed; }
    public int getAngle(){ return angle; }
    public Rectangle getRect(){
        return new Rectangle(getX()-radius, getY()-radius, radius*2, radius*2);
    }
    // set
    public void setRadius(int radius){
        if((radius >= Const.BALL_MIN_RADIUS) && (radius <= Const.BALL_MAX_RADIUS)) this.radius = radius;
    }
    public void setFixed(boolean fixed){
        this.fixed = fixed;
    }
    public void setPaddle(Paddle paddle){
        this.paddle = paddle;
    }
    public void setCalcdata(double x, double y){
        xcalc = x; ycalc = y;
    }
    public void setSpeed(double speed){
        if((speed >= Const.BALL_MIN_SPEED) && (speed <= Const.BALL_MAX_SPEED)) this.speed = speed;
    }
    public void setAngle(int angle){
        xdir = Math.cos(Math.toRadians(angle));
        ydir = Math.sin(Math.toRadians(angle));
        this.angle = angle;
        if(this.angle>360){ this.angle-=360; }
        else if(this.angle<0){ this.angle+=360; }
    }
    // other
    public void resetState(){
        //setCalcdata(startx, starty);
        //setPosition(startx, starty);
        setAngle(Const.START_BALL_ANGLE);
        setSpeed(Const.BALL_SPEED);
        setRadius(Const.BALL_RADIUS);
    }
    public void move(){
        if(!fixed){
            setCalcdata(xcalc+xdir*speed, ycalc+ydir*speed);
            setPosition((int)xcalc, (int)ycalc);
        }
        else{
            setPosition(paddle.getX(), paddle.getY()-(radius+paddle.getHeight()/2));
            setCalcdata(getX(), getY());
        }
    }
    public void draw(Graphics g){
        g.setColor(getColor());
        g.fillOval(getX()-radius, getY()-radius, radius*2, radius*2);
        g.setColor(new Color(255,255,255, 168));
        g.fillOval(getX()-radius+radius/2, getY()-radius+radius/2, radius/2, radius/2);
    }

    private void reflection(int side){
        switch(side){
            case 0: case 2:
                setAngle(-angle); break;
            case 1: case 3:
                setAngle(-angle+180); break;
        }
    }
    public void changeDir(int side, int relativex) {
        int change;
        changeDir(side);
        if(relativex!=0){
            change = (int)(40.0*((double)relativex/(double)(paddle.getWidth()/2)));
            if((change<0) && (angle>90)) setAngle(angle+change);
            if((change>0) && (angle>90)) setAngle(angle+change);
        }
    }
    public void changeDir(int side){
        int randomness = rand.nextInt(10)-5;
        switch(side){
            case 0: if(ydir<0) reflection(0); break;
            case 2: if(ydir>0) reflection(2); break;
            case 1: if(xdir>0) reflection(1); break;
            case 3: if(xdir<0) reflection(3); break;
        }
        setAngle(angle+randomness);
    }
}
